/****************************************************************************
 *                                                                          *
 * U U    6   1            U U   FFF  O   O  TTT                            *
 * U U   6   11   b        U U   F   O O O O  T                             *
 * U U - 66   1   bb  y y  U U - FF  O O O O  T                             *
 * U U   6 6  1   b b  y   U U   F   O O O O  T                             *
 *  U     6   1   bb   y    U    F    O   O   T                             *
 *                                                                          *
 * U61 is another block based game                                          *
 * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
 *                                                                          *
 * This program is free software; you can redistribute it and/or            *
 * modify it under the terms of the GNU General Public License              *
 * as published by the Free Software Foundation; either version 2           *
 * of the License, or (at your option) any later version.                   *
 *                                                                          *
 * This program is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU General Public License for more details.                             *
 *                                                                          *
 * You should have received a copy of the GNU General Public License        *
 * along with this program; if not, write to the Free Software              *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
 *                                                                          *
 * This project is also available on SourceForge  (http://sourceforge.net)  *
 ****************************************************************************/

/*
 * file name:   graphicsmenu.cpp
 * author:      U-Foot (ufoot@ufoot.org / www.ufoot.org)
 * description: this menu lets the user set the graphics parameters,
 *              such as the maximum number of frames per second
 */

/*---------------------------------------------------------------------------
 includes
 ---------------------------------------------------------------------------*/

#include "graphicsmenu.h"
#include "number.h"
#include "global.h"
#include "button.h"
#include "utils.h"

/*---------------------------------------------------------------------------
 globals
 ---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------
 functions
 ---------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* 
 * creation of the sound menu
 */ 
U61_GraphicsMenu::U61_GraphicsMenu(U61_Menu *parent,
				   U61_Config *conf)
  : U61_Menu(parent,"Graphics options")
{
  config=conf;

  add_item(new U61_Button("Toggle fullscreen"));
  add_item(new U61_Number("Max frames per sec",
             &(conf->max_fps),5,50));

  select(0);
}

/*--------------------------------------------------------------------------*/
/*
 * validate is called when the user presses enter
 */
U61_Menu *U61_GraphicsMenu::validate()
{
  U61_Menu *next_menu;

  next_menu=this;
  switch (selection)
    {
    case 0:
      if (U61_Utils::set_videomode(U61_Global::data.screen_w,
				   U61_Global::data.screen_h,
				   !config->fullscreen))
	{
	  config->fullscreen=!config->fullscreen;
	  U61_Global::input_monitor.reset();
	}
      break;
    }

  return next_menu;
}

